ATMTracker = LibStub("AceAddon-3.0"):NewAddon("ATMTracker", "AceConsole-3.0", "AceEvent-3.0")

local modules = {
    [1] = {
		itemId = 118375,
		location = "Invasions & Raids",
		parts = {
			[1] = {
				itemId = 118344,
			},
			[2] = {
				itemId = 118345,
			},
			[3] = {
				itemId = 118346,
			},
			[4] = {
				itemId = 118347,
			},			
		},
	},
    [2] = {
		itemId = 118376,
		location = "Dungeons",
		parts = {
			[1] = {
				itemId = 118332,
			},
			[2] = {
				itemId = 118331,
			},
			[3] = {
				itemId = 118197,
			},
		},
	},
    [3] = {
		itemId = 118377,
		location = "Ashran",
		parts = {
			[1] = {
				itemId = 118333,
			},
			[2] = {
				itemId = 118334,
			},
			[3] = {
				itemId = 118335,
			},	
		},
	},
    [4] = {
		itemId = 118378,
		location = "Shadowmoon, Talador, and Nagrand",
		parts = {
			[1] = {
				itemId = 118336,
			},
			[2] = {
				itemId = 118337,
			},
			[3] = {
				itemId = 118338,
			},
			[4] = {
				itemId = 118339,
			},			
		},
	},
    [5] = {
		itemId = 118379,
		location = "Frostfire, Gorgrond, and Spires of Arak",
		parts = {
			[1] = {
				itemId = 118340,
			},
			[2] = {
				itemId = 118341,
			},
			[3] = {
				itemId = 118342,
			},
			[4] = {
				itemId = 118343,
			},			
		},
	},
}

local options = {
	name = "ATMTracker",
	handler = ATMTracker,
	type = "group",
	args = {
		version = {
			order = 0,
			type = "description",
			name = "v1.1",
			width = "full",
		},
		desc = {
			order = 1,
			type = "description",
			name = "Track What Ancient Trading Mechanism Parts Have Been Collected For Garrison Trading Post Auction House Function",
			width = "full",
		},
		showInChat = {
			order = 4,
            type = "toggle",
            name = "Show in Chat",
            desc = "Toggles the display of the message in the chat window.",
            get = "IsShowInChat",
            set = "ToggleShowInChat",
        },
        showOnScreen = {
			order = 5,
            type = "toggle",
            name = "Show on Screen",
            desc = "Toggles the display of the message on the screen.",
            get = "IsShowOnScreen",
            set = "ToggleShowOnScreen",
		},
		showMinimapIcon = {
			order = 6,
            type = "toggle",
            name = "Hide Minimap Icon",
            desc = "Toggles the display of the minimap icon.",
            get = "IsShowMinimapIcon",
            set = "ToggleShowMinimapIcon",
		},
	},
}

local ldb = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("ATMTLDB", {
	type = "launcher",
	icon = "Interface\\Addons\\ATMTracker\\icon",
	tocname = "ATMTracker",
	OnClick = function(clickedframe, button)
		if button == "RightButton" then ATMTracker:ShowConfig() else ATMTracker:ShowATMT() end
	end,		
	OnTooltipShow = function(tt)
		tt:AddLine("|cffffffff" .. options.name .. "|r")
		tt:AddLine("Click to open ATMTracker.")
		tt:AddLine("Right-click to open ATMT config.")
		end,		
	})

local defaults = {
	profile = {
		showInChat = false,
		showOnScreen = true,
		minimapIcon = {
			hide = false,
		},
	},
}

function ATMTracker:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ATMTrackerDB", defaults, true);
	icon = LibStub("LibDBIcon-1.0");
	AceGUI = LibStub("AceGUI-3.0");
	tip = LibStub('LibQTip-1.0'):Acquire('MyFooBarTooltip', 5, "LEFT", "CENTER");
    LibStub("AceConfig-3.0"):RegisterOptionsTable("ATMTracker", options);
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ATMTracker", "ATMTracker");
	icon:Register("ATMTLDB", ldb, self.db.profile.minimapIcon);
	self:RegisterChatCommand("atmt", "ChatCommand");
end;

function ATMTracker:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
end;

function ATMTracker:ShowATMT()
	local frame = AceGUI:Create("Frame");
	frame:SetCallback("OnClose",function(widget) AceGUI:Release(widget) end);
	frame:SetTitle("Ancient Trading Mechanism Tracker");
	frame:SetLayout("Flow");
	frame:SetWidth(800);	
	frame:SetHeight(600);
	for j=1,table.getn(modules),1 do
		local heading = AceGUI:Create("Heading");
		heading:SetText(ATMTracker:GetItemName(modules[j].itemId));
		heading:SetRelativeWidth(1.0);
		frame:AddChild(heading);
		local location = AceGUI:Create("Label");
		location:SetText(modules[j].location);
		location:SetRelativeWidth(1.0);	
		frame:AddChild(location);
		local module = AceGUI:Create("InteractiveLabel");
		module:SetImage(ATMTracker:GetItemTexture(modules[j].itemId));
		module:SetText(ATMTracker:GetItemName(modules[j].itemId));
		module:SetImageSize(40,40);
		if ATMTracker:IsModuleCompleted(j) then
			module:SetHighlight(0,255,0,1);
		else
			module:SetHighlight(255,0,0,1);
		end
		module:SetWidth(130);
		frame:AddChild(module);
		local space = AceGUI:Create("Label");
		space:SetText("");
		space:SetWidth(40);
		frame:AddChild(space);
		for i=1,table.getn(modules[j].parts),1 do
			local part = AceGUI:Create("InteractiveLabel");
			part:SetImage(ATMTracker:GetItemTexture(modules[j].parts[i].itemId));
			part:SetText(ATMTracker:GetItemName(modules[j].parts[i].itemId));
			part:SetImageSize(40,40);
			part:SetCallback("OnEnter", ATMTracker:ShowTooltip(modules[j].parts[i].itemId))
			part:SetCallback("OnLeave", ATMTracker:HideTooltip())
			if (GetItemCount(modules[j].parts[i].itemId, true) == 0) and not (ATMTracker:IsModuleCompleted(j)) then
				part:SetHighlight(255,0,0,1);
			else
				part:SetHighlight(0,255,0,1);
			end
			part:SetWidth(130);
			frame:AddChild(part);
		end;
	end;
end;

function ATMTracker:ShowTooltip(itemId)

end

function ATMTracker:HideTooltip()

end

function ATMTracker:OnEnable()
	self:RegisterEvent("CHAT_MSG_LOOT","OnEvent");
end;

function ATMTracker:OnEvent(...)
	local event, chat_msg, arg2, arg3, arg4, target = select(1, ...);
	if ((event == "CHAT_MSG_LOOT") and (string.find(chat_msg, "You receive loot"))) then
        local _,_,itemId = string.find(string.match(chat_msg, "You receive loot: (.+)%."), ".*|Hitem:(%d+):.*");
		itemId = tonumber(itemId);
		if ATMTracker:IsPartLooted(itemId) then
			ATMTracker:PostMessage(itemId);
		end;
	end;
end;

function ATMTracker:IsShowInChat(info)
    return self.db.profile.showInChat;
end;

function ATMTracker:ToggleShowInChat(info, value)
    self.db.profile.showInChat = value;
end;

function ATMTracker:IsShowMinimapIcon(info)
    return self.db.profile.minimapIcon.hide ;
end;

function ATMTracker:ToggleShowMinimapIcon(info, value)
	if ATMTracker:IsShowMinimapIcon(info) then
		icon:Show("ATMTLDB")
	else
		icon:Hide("ATMTLDB")
	end
    self.db.profile.minimapIcon.hide = value;
end;

function ATMTracker:IsShowOnScreen(info)
    return self.db.profile.showOnScreen;
end;

function ATMTracker:ToggleShowOnScreen(info, value)
    self.db.profile.showOnScreen = nvalue;
end;

function ATMTracker:ChatCommand(input)
	if not input or input:trim() == "" then
		ATMTracker:ShowConfig()
	else
		LibStub("AceConfigCmd-3.0"):HandleCommand("atmt", "ATMTracker", input)
    end;
end;

function ATMTracker:GetItemLink(itemId)
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId);
	return(itemLink);
end;

function ATMTracker:GetItemTexture(itemId)
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId);
	return(itemTexture);
end;

function ATMTracker:GetItemName(itemId)
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId);
	return(itemName);
end;

function ATMTracker:GetModule(itemId)
	for j=1,table.getn(modules),1 do
		for i=1,table.getn(modules[j].parts),1 do
			if (modules[j].parts[i].itemId == itemId) then
				return j;
			end;
		end;
	end;
end;

function ATMTracker:IsPartLooted(itemId)
	for j=1,table.getn(modules),1 do
		for i=1,table.getn(modules[j].parts),1 do
			if (modules[j].parts[i].itemId == itemId) then
				return true;
			end;
		end;
	end;
	return false;
end;

function ATMTracker:PartsCompleted(j)
	count = 0;
	for i=1,table.getn(modules[j].parts),1 do
		if (GetItemCount(modules[j].parts[i].itemId, true) ~= 0) then
			count = count + 1;
		end;
	end;
	return count;
end;

function ATMTracker:IsModuleCompleted(j)
	if (GetItemCount(modules[j].itemId, true) > 0) then
		return true;
	else
		local completed = true;
			for i=1,table.getn(modules[j].parts),1 do
				if (GetItemCount(modules[j].parts[i].itemId, true) == 0) then
					completed = false;
				end;
			end;
		return completed;
	end;
end;

function ATMTracker:GetMessage(itemId)
	local partCount = GetItemCount(itemId, true)+1;
	local supportmodule = ATMTracker:GetModule(itemId);
	local moduleCount = GetItemCount(itemId, true)+1;
	local message = "";
	if partCount == 1 then
		message = "Inventory: " .. ATMTracker:GetItemLink(itemId);
	elseif partCount > 1 then
		message = "Inventory: " .. ATMTracker:GetItemLink(itemId) .. "x " .. partCount;
	end;
	if ATMTracker:IsModuleCompleted(supportmodule) then
		if moduleCount > 1 then
			message = message .."\nInventory: " .. ATMTracker:GetItemLink(modules[supportmodule].itemId) .. "x " .. moduleCount;
		else 
			message = message .."\nInventory: " .. ATMTracker:GetItemLink(modules[supportmodule].itemId);
		end;
	else
		message = message .. "\nInventory: " .. (ATMTracker:PartsCompleted(supportmodule)+1) .. "of" .. table.getn(modules[supportmodule].parts) .. "Collected" .. ATMTracker:GetItemLink(modules[supportmodule].itemId)	
	end;
	return message;
end;

function ATMTracker:PostMessage(itemId)
	local message = ATMTracker:GetMessage(itemId);
    if self.db.profile.showInChat then
        ATMTracker:Print("\n" .. message);
    end;
    if self.db.profile.showOnScreen then
        UIErrorsFrame:AddMessage(message, 1.0, 1.0, 1.0, 5.0);
    end;
end;

function ATMTracker:ModuleCompleted(input)
	local completed = true;
	for i=1,table.getn(modules[input].parts),1 do
		if (GetItemCount(modules[input].parts[i].itemId, true) == 0) then
			completed = false;
		end;
	end;
	return completed;
end;

function ATMTracker:InitializeLinks()
	for j=1,table.getn(modules),1 do
		for i=1,table.getn(modules[j].parts),1 do
			self:GetItemLink(modules[j].parts[i].itemId, true);
		end;
	end;
end;
